import requests
import json
import numpy as np

supportedCurrencies = []


def notValidCurrencyError():
    raise Exception(f"Currency not supported \n Supported currencies are: {supportedCurrencies}")

class Currency:
    def __init__(self, shorthand, amount):
        self.shorthand = shorthand
        self.amount = amount
        self.date = None

        if verifyShorthand(self.shorthand) == False:
            notValidCurrencyError()




def Convert(base, shorthandToConvert):


    if verifyShorthand(shorthandToConvert) == False:
        notValidCurrencyError()

    req = requests.get(f"https://api.exchangeratesapi.io/latest?base={base.shorthand}")
    data = req.json()

    converted = base.amount * data["rates"][shorthandToConvert]
    obj = Currency(shorthandToConvert, converted)
    obj.date = data["date"]

    return obj

def updateSupportedCurrencies():
    global supportedCurrencies

    supportedCurrencies = []

    req = requests.get("https://api.exchangeratesapi.io/latest")
    data = req.json()

    keys = data["rates"].keys()
    array = list(keys)

    for x in range(0, len(array)):
        supportedCurrencies.append(array[x])

    supportedCurrencies.append("EUR")

def verifyShorthand(toVerify):
    if toVerify in supportedCurrencies:
        return True
    else:
        return False



updateSupportedCurrencies()


import StockStalker.utils as utils
from StockStalker.present import getLiveData

import pandas as pd
import threading
import time
import os
import logging

class StalkerCallback:
    def __init__(self, function=None):
        self.func = function
        #self.args = arg

class Stalker:
    def __init__(self, target=None, duration=None, verbose=True, name="Bob", long_poll_delay=30, save=True, save_location="saves", dailyCallback=None, doneCallback=None):
        self.target = target
        self.duration = duration
        self.verbose = verbose
        self.name = name
        self.pollDelay = long_poll_delay
        self.save = save
        self.saveLoc = save_location

        self.dailyCallback = dailyCallback
        self.doneCallback = doneCallback

        self.df = pd.DataFrame(columns=["Symbol","Price","Change","P/E","Ranges","Open","previousClose","EPS","marketCap","volume"])

        self.thread = None
        self.__killed = False

    def __update(self):
        newData = None

        try:
            newData = getLiveData(self.target, format="DataFrame")
        except:
            return

        length = len(self.df.index)

        newData = newData.values.tolist()

        self.df.loc[length+1] = newData[0]

        print("Updated Values")

    def __save(self):
        if os.path.exists(self.saveLoc) == False:
            os.mkdir(self.saveLoc)
        print(f"Stalker '{self.name}' saved to file")
        self.df.to_csv(f"{self.saveLoc}/{self.name}.csv")


    def __run(self):
        for x in range(0, self.duration):

            if self.__killed:
                break

            if self.verbose:
                print(f"Stalker '{self.name}' began polling for market open")

            while utils.marketOpen() == False:
                if self.verbose:
                    print(f"Stalker '{self.name}' polled for market open")
                if self.__killed:
                    break

                time.sleep(self.pollDelay)

            self.__update()

            if self.save:
                self.__save()

            if self.dailyCallback != None:
                self.dailyCallback.func()

            print(f"Stalker '{self.name}' is waiting for the next day")

            duration = 18000

            for y in range(0, duration):
                if self.__killed:
                    break
                time.sleep(1)


        if self.doneCallback != None:
            self.doneCallback.func()


    #def loadParams(self, file):


    def run(self):

        self.thread = threading.Thread(target=self.__run)
        self.thread.start()

    def wait(self):
        self.thread.join()
    def kill(self):
        self.__killed = True


import datetime
import time
import requests
import pandas as pd

date_format = "%m/%d/%Y"

def now():
    return datetime.datetime.utcnow()

def prettyDate(dt):
    return datetime.datetime.strftime(dt, date_format)


def futureError(date):
    raise Exception(f"Supplied Date ({date}) is in the future")


def toUnixStamp(strDate):
    #return datetime.datetime.strptime(strDate, date_format).timestamp()
    return int(pd.Timestamp(strDate).timestamp())




def getPastData(ticker, startDate, toDate, interval="d"):

    start = datetime.datetime.strptime(startDate, date_format)
    end = datetime.datetime.strptime(toDate, date_format)

    current = datetime.datetime.strptime(prettyDate(now()), date_format)

    if start > current:
        futureError(startDate)
    if start > current:
        futureError(startDate)

    if end > start:
        raise Exception("Start date cant be less than destination date")


    if interval not in ["d", "w", "mo"]:
        raise Exception("Interval must be day (d), week (w) or month (mo)")

    startUnix = toUnixStamp(startDate)
    endUnix = toUnixStamp(toDate)


    url = f"https://query1.finance.yahoo.com/v7/finance/download/{ticker}?period1={endUnix}&period2={startUnix}&interval=1d&events=history&incl"
    #print(url)

    df = pd.read_csv(url)

    return df




import bs4
import requests
import re
import StockStalker.present as present
import pandas as pd
from munch import munchify

def getCrypto(cryptoTicker, format="Dict"):
    try:
        page = requests.get(f"https://finance.yahoo.com/quote/{cryptoTicker}-USD")
    except:
        raise Exception(f"Ticker '{cryptoTicker}' does exist / cannot get page")

    soup = bs4.BeautifulSoup(page.content, 'html.parser')


    price = getPrice(soup)

    getChanges = getChange(soup)
    numChange = getChanges[0]
    percChange = getChanges[1]

    ranges = getRange(soup)
    low = ranges[0]
    high = ranges[1]

    open = present.getOpen(soup)
    close = present.getPrevClose(soup)

    dataDict = {
        "Symbol": cryptoTicker,
        "Currency": "USD",
        "Price": price,
        "Change": [{
            "change": numChange,
            "percentageChange": percChange
        }],
        "Ranges": [{
            "Low": low,
            "High": high
        }],
        "Open": open,
        "previousClose": close,
    }

    if format == "Dict":
        return dataDict
    if format == "DataFrame":
        return pd.DataFrame(dataDict)
    if format == "Object":
        return munchify(dataDict)

    raise Exception("Invalid Format Option")




def getPrice(soup):
    hits = soup.find(class_="Trsdu(0.3s) Fw(b) Fz(36px) Mb(-4px) D(ib)")

    if "," in hits.text:

        hits = hits.text.replace(",", "")

    try:
        return float(hits)
    except:
        return "N/A"



def getChange(soup):
    hits = soup.find(class_='Trsdu(0.3s) Fw(500) Pstart(10px) Fz(24px) C($negativeColor)')

    if hits == None:
        hits = soup.find(class_='Trsdu(0.3s) Fw(500) Pstart(10px) Fz(24px) C($positiveColor)')

    if hits == None:
        return "N/A"

    try:
        numChange = hits.text.split(" ")[0].replace("+", "")
        percChange = re.search(r'\((.+)\)', hits.text).group(0).split("(")[1].split(")")[0]
    except:
        return "N/A"



    return [float(numChange), percChange]


def getRange(soup):
    hits = soup.find(attrs={"data-test": "DAYS_RANGE-value"})

    if hits == None:
        return "N/A"

    split = hits.text.split("-")

    try:

        if "," in split[0]:
           low = float(split[0].replace(",", ""))
        else:
            low = float(split[0])
        if "," in split[1]:
            high = float(split[1].replace(",", ""))
        else:
            high = float(split[1])

    except:
        return "N/A"
    return [low, high]





import requests
import bs4
import datetime


def marketOpen():

    primaryCheck = False

    weekday = datetime.datetime.today().weekday()
    currentTime = datetime.datetime.utcnow()
    hour = currentTime.hour

    if weekday < 5 and hour in range(15, 21):
        primaryCheck = True



    if primaryCheck:
        page = requests.get("https://www.tradinghours.com/open?")
        soup = bs4.BeautifulSoup(page.content, 'html.parser')
        hit = soup.find(class_='text-open font-weight-bold')
        if hit == None:
            hit = soup.find(class_='text-close font-weight-bold')

        if hit.text == "Yes":
            return True

        else:
            return False

    return False


from setuptools import setup



setup(name='StockStalker',
      version='0.1',
      description='easy to use, robust and quick stocks package',
      long_description="README.md",
      long_description_content_type='text/markdown',
      url='https://github.com/RockoonTechnologies/StockStalker',
      author='Meepers',
      author_email='rockoontech@gmail.com',
      license='MIT',
      packages=['StockStalker'],
      install_requires=[
          'beautifulsoup4==4.9.3',
          'pandas==1.1.5',
          'requests==2.25.1',
          'lxml==4.6.2',
          'Munch==2.5.0'
      ],
      zip_safe=False)



import requests
import bs4
import re
import pandas as pd
from ftplib import FTP
from munch import munchify


import os
import shutil

if os.path.exists("cache") == False:
    os.mkdir("cache")
else:
    shutil.rmtree("cache")




def getLiveData(ticker, format="Dict"):
    page = None
    try:
        page = requests.get(f"https://finance.yahoo.com/quote/{ticker}")
    except:
        raise Exception(f"Ticker '{ticker}' does exist / cannot get page")

    soup = bs4.BeautifulSoup(page.content, 'html.parser')


    price = getPrice(soup)

    changes = getChange(soup)
    numChange = changes[0]
    percChange = changes[1]

    pe = getPEttm(soup)

    ranges = getRange(soup)
    low = ranges[0]
    high = ranges[1]

    open = getOpen(soup)
    prevClose = getPrevClose(soup)

    eps = getEPS(soup)

    mktCap = getMktCap(soup)

    volume = getVolume(soup)


    dataDict = {
        "Symbol": ticker,
        "Price": price,
        "Change": [{
            "change": numChange,
            "percentageChange": percChange
        }],
        "P/E": pe,
        "Ranges": [{
            "Low": low,
            "High": high
        }],
        "Open": open,
        "previousClose": prevClose,
        "EPS": eps,
        "marketCap": mktCap,
        "volume": volume
    }

    if format == "Dict":
        return dataDict
    if format == "DataFrame":
        return pd.DataFrame(dataDict)
    if format == "Object":
        return munchify(dataDict)

    raise Exception("Invalid Format Option")




def getPrice(soup):
    hits = soup.find(class_='Trsdu(0.3s) Fw(b) Fz(36px) Mb(-4px) D(ib)')

    if hits==None:
        return "N/A"

    if "," in hits.text:
        hits = hits.text.replace(",", "")
    else:
        hits = hits.text

    try:
        return float(hits)
    except:
        return "N/A"


def getChange(soup):
    hits = soup.find(class_='Trsdu(0.3s) Fw(500) Pstart(10px) Fz(24px) C($negativeColor)')

    if hits == None:
        hits = soup.find(class_='Trsdu(0.3s) Fw(500) Pstart(10px) Fz(24px) C($positiveColor)')

    if hits == None:
        return "N/A"

    try:
        numChange = hits.text.split(" ")[0].replace("+", "")
        percChange = re.search(r'\((.+)\)', hits.text).group(0).split("(")[1].split(")")[0]
    except:
        return "N/A"

    if "," in numChange:
        numChange = numChange.replace(",", "")

    return [float(numChange), percChange]

def getPEttm(soup):

    hits = soup.find(attrs={"data-reactid":149})

    if hits == None:
        return "N/A"

    val = hits.text.replace(",", "")

    try:
        return float(val)
    except:
        return val

def getRange(soup):
    hits = soup.find(attrs={"data-reactid": 117})
    if hits == None:
        return "N/A"

    split = hits.text.split("-")
    try:
        low = float(split[0])
        high = float(split[1])
    except:
        return "N/A"
    return [low, high]


def getOpen(soup):
    hits = soup.find(attrs={"data-test": "OPEN-value"})

    if hits == None:
        return "N/A"

    if "," in hits.text:
        hits = hits.text.replace(",", "")
        final = float(hits)


    try:
        final = float(hits)
    except:
        try:
            final = float(hits.text)
        except:
            return "N/A"
    return final

def getPrevClose(soup):
    hits = soup.find(attrs={"data-test": "PREV_CLOSE-value"})
    if hits == None:
        return "N/A"

    if "," in hits.text:
        hits = hits.text.replace(",", "")
        final = float(hits)
    else:
        final = float(hits.text)


    return final

def getEPS(soup):
    hits = soup.find(attrs={"data-test": "EPS_RATIO-value"})
    try:
        final = float(hits.text)
    except:
        return "N/A"
    return final

def getMktCap(soup):
    hits = soup.find(attrs={"data-test": "MARKET_CAP-value"})
    try:
        final = str(hits.text)
    except:
        return "N/A"
    if "B" in final:
        final = final.replace("B", "")
        final = float(final)
        final *= 1000000000
    elif "M" in final:
        final = final.replace("M", "")
        final = float(final)
        final *= 1000000


    return final

def getVolume(soup):
    hits = soup.find(attrs={"data-test": "TD_VOLUME-value"})
    if hits == None:
        return "N/A"

    final = hits.text.replace(",", "", -1)
    final = float(final)

    return final





def getIndex(index):
    if index == "S&P500":
        page = pd.read_html("https://en.wikipedia.org/wiki/List_of_S%26P_500_companies")
        return page[0]['Symbol'].tolist()

    if index == "NASDAQ":
        #ftp://ftp.nasdaqtrader.com/SymbolDirectory/nasdaqlisted.txt

        ftp = FTP('ftp.nasdaqtrader.com')
        ftp.login()
        ftp.cwd("SymbolDirectory")

        file = ftp.nlst()[9]

        ftp.retrbinary("RETR " + file, open("cache/nasdaq.txt", 'wb').write)

        df = pd.read_csv("cache/nasdaq.txt", sep="|")["Symbol"].tolist()
        return df

    if index == "NASDAQ.OTHER":
        ftp = FTP('ftp.nasdaqtrader.com')
        ftp.login()
        ftp.cwd("SymbolDirectory")
        file = ftp.nlst()[13]

        ftp.retrbinary("RETR " + file, open("cache/other.txt", 'wb').write)
        df = pd.read_csv("cache/other.txt", sep="|")["NASDAQ Symbol"].tolist()
        return df

    if index == "DOW":
        #https://en.wikipedia.org/wiki/Dow_Jones_Industrial_Average
        page = pd.read_html("https://finance.yahoo.com/quote/%5EDJI/components?p=%5EDJI")[0]["Symbol"].tolist()

        return page

    if index == "NYSE":
        ftp = FTP('ftp.nyse.com')
        ftp.login()
        ftp.cwd("NYSESymbolMapping")
        file = ftp.nlst()[0]

        ftp.retrbinary("RETR " + file, open("cache/nyse.txt", 'wb').write)

        df = pd.read_csv("cache/nyse.txt", sep="|").iloc[:, 0].tolist()
        return df


